"""
Embedded Python Blocks:

Each time this file is saved, GRC will instantiate the first class it finds
to get ports and parameters of your block. The arguments to __init__  will
be the parameters. All of them are required to have default values!
"""

import numpy as np
from gnuradio import gr


class sinad(gr.sync_block):
    """
    docstring for block sinad
    """

    def __init__(self, sinadFreq=1750, Fs=48000, fft_size=4096):
        gr.sync_block.__init__(self, "sinad", [np.float32], [np.float32])
        self.Fs = Fs
        self.fRef = sinadFreq
        self.refWidth = 20
        self.fft_size = fft_size
        # effectuer les calculs par groupe d'échantillons égal à la taille de FFT
        self.set_output_multiple(fft_size)
 
    def work(self, input_items, output_items):
        output_items[0][:] = self.__calc_sinad(input_items[0])
        return len(output_items[0])

    def __calc_sinad(self, data):
        """ Calcul du SINAD en dB
        """
        # Calculer la FFT

        window = np.hamming(np.size(data))
        data = data * window

        psd = np.fft.fft(data)

        # Determiner les index des bins
        n = len(psd)
        #Le signal se situe entre bin1 et bin2
        bin1 = int((self.fRef - self.refWidth / 2) / self.Fs * n)
        bin2 = int((self.fRef + self.refWidth / 2) / self.Fs * n)

        # limitation de la bande passante : 300 Hz - 3000Hz

        bin300hz = int(300 / self.Fs * n)
        bin3000hz = int(3000 / self.Fs * n)

        # SINAD = 10*log10((Ps + Pn) / Pn)

        signal = psd[bin300hz:bin3000hz]
        noise = np.concatenate((psd[bin300hz:bin1], psd[(bin2 + 1):bin3000hz]))

        Ps = np.sum(np.abs(signal) ** 2)
        Pn = np.sum(np.abs(noise) ** 2)

        sinad = max(10 * np.log10(Ps / Pn), 0) if Ps > 0 else 0
        return sinad